//
//
// RevBytes.c
//
// Code taken from something Chris Torek posted to c.l.c.
// He called it the "butterfly shift" :)
//
//

#include <stdlib.h>
#include <stdio.h>

unsigned long long reverse_bytes_64(unsigned long long tmp)
{
	tmp = ((tmp >>  8) & 0x00ff00ff00ff00ffULL) | ((tmp <<  8) & 0xff00ff00ff00ff00ULL);
	tmp = ((tmp >> 16) & 0x0000ffff0000ffffULL) | ((tmp << 16) & 0xffff0000ffff0000ULL);
	tmp = ((tmp >> 32) & 0x00000000ffffffffULL) | ((tmp << 32) & 0xffffffff00000000ULL);
	return tmp;
}

unsigned long reverse_bytes_32(unsigned long tmp)
{
	tmp = ((tmp >>  8) & 0x00ff00ffUL) | ((tmp <<  8) & 0xff00ff00UL);
	tmp = ((tmp >> 16) & 0x0000ffffUL) | ((tmp << 16) & 0xffff0000UL);
	return tmp;
}

unsigned short reverse_bytes_16(unsigned short tmp)
{
	tmp = ((tmp >> 8) & 0x00ff) | ((tmp << 8) & 0xff00);
	return tmp;
}

unsigned long long reverse_bits_64(unsigned long long tmp)
{
	tmp = ((tmp >> 1) & 0x5555555555555555ULL) | ((tmp << 1) & 0xaaaaaaaaaaaaaaaaULL);
    tmp = ((tmp >> 2) & 0x3333333333333333ULL) | ((tmp << 2) & 0xccccccccccccccccULL);
	return tmp;
}

unsigned long reverse_bits_32(unsigned long tmp)
{
	tmp = ((tmp >> 1) & 0x55555555L) | ((tmp << 1) & 0xaaaaaaaaL);
    tmp = ((tmp >> 2) & 0x33333333L) | ((tmp << 2) & 0xccccccccL);
	return tmp;
}

unsigned short reverse_bits_16(unsigned short tmp)
{
	tmp = ((tmp >> 1) & 0x5555) | ((tmp << 1) & 0xaaaa);
    tmp = ((tmp >> 2) & 0x3333) | ((tmp << 2) & 0xcccc);
	return tmp;
}

unsigned char reverse_bits_8(unsigned char tmp)
{
	tmp = ((tmp >> 1) & 0x55) | ((tmp << 1) & 0xaa);
    tmp = ((tmp >> 2) & 0x33) | ((tmp << 2) & 0xcc);
	return tmp;
}

int main(void)
{

    unsigned long long Int64 = 0x1a2a3a4a5a6a7a8aULL;
    unsigned long      Int32 = 0x1a2a3a4aUL;
    unsigned short     Int16 = 0x1a2a;
	unsigned char 	   Int8  = 0xa5;
	char mstr[17];

	printf("Int64 Bytes \t%s\n", _ui64toa(Int64, mstr, 16));
	_ui64toa(reverse_bytes_64(Int64), mstr, 16);
	printf("Reverse Bytes \t%s\n", mstr);

	printf("\n");

	printf("Int32 Bytes \t%x\n", Int32);
	Int32 = reverse_bytes_32(Int32);
	printf("Reverse Bytes \t%x\n", Int32);

	printf("\n");

	printf("Int16 Bytes \t%x\n", Int16);
	Int16 = reverse_bytes_16(Int16);
	printf("Reverse Bytes \t%x\n", Int16);

	printf("\n");

    Int64 = 0xa5a5a5a5a5a5a5a5ULL;
	printf("Int64 Bits \t%s\n", _ui64toa(Int64, mstr, 16));
	Int64 = reverse_bits_64(Int64);
	printf("Reverse Bits \t%s\n", _ui64toa(Int64, mstr, 16));

	printf("\n");

	Int32 = 0xa5a5a5a5UL;
	printf("Int32 Bits \t%8x\n", Int32);
	Int32 = reverse_bits_32(Int32);
	printf("Reverse Bits \t%8x\n", Int32);

	printf("\n");

	Int16 = 0xa5a5;
	printf("Int16 Bits \t%4x\n", Int16);
	Int16 = reverse_bits_16(Int16);
	printf("Reverse Bits \t%4x\n", Int16);

	printf("\n");

	printf("Int8 Bits \t%x\n", Int8);
	Int8 = reverse_bits_8(Int8);
	printf("Reverse Bits \t%x\n", Int8);

	printf("\nPress Return\n");
	fgets(mstr, 16, stdin);

	return 0;
}
